@@ -8,7 +8,7 @@ LOGGER = logging.getLogger("azure")
 LOGGER.setLevel(logging.WARN)
 
 
-def build_user_object(user_details: dict, extension_id: str, b2c_prefix: str) -> dict:
+def build_local_user_object(user_details: dict, b2c_prefix: str, extension_id: str) -> dict:
     """Builds a user object to be used in the creation or update of a user in Azure AD B2C"""
     if user_details['role'] == '8amAdmin':
         user_details['password'] = 'eightamAdmin!23'
@@ -38,8 +38,35 @@ def build_user_object(user_details: dict, extension_id: str, b2c_prefix: str) ->
     }
 
 
+def build_federated_user_object(user_details: dict, extension_id: str) -> dict:
+    """Builds a federated user object to be used in the creation or update of a user in Azure AD B2C"""
+    return {
+        "accountEnabled": True,
+        "displayName": f"{user_details['firstName']} {user_details['lastName']}",
+        "mailNickname": user_details['firstName'],
+        "otherMails": [user_details['email']],
+        "passwordProfile": {
+            "forceChangePasswordNextSignIn": False
+        },
+        "mobilePhone": user_details['phone'],
+        f"extension_{extension_id}_OrganizationID": user_details['company_id'],
+        f"extension_{extension_id}_SignUpCode": user_details['sign_up_code'],
+        f"extension_{extension_id}_UserRoles": user_details['role'],
+        f"extension_{extension_id}_mustResetPassword": user_details['reset_password']
+    }
+
+
+def build_user_object(user_details: dict, extension_id: str, b2c_prefix: str) -> dict:
+    """Builds a user object to be used in the creation or update of a user in Azure AD B2C"""
+    if ('signInType' in user_details) and (user_details['signInType'] == 'federated'):
+        return build_federated_user_object(user_details, extension_id)
+    else:
+        return build_local_user_object(user_details, b2c_prefix, extension_id)
+
+
 class B2CHelper:
     """Class providing functionality to interact with Azure AD B2C via the Microsoft Graph API"""
+
     def __init__(self, token_request_data) -> None:
         self.token_request_data = {
             'grant_type': token_request_data['grant_type'],
@@ -50,17 +77,15 @@ class B2CHelper:
             'extension_id': token_request_data['ext_app_client_id']
         }
 
-
     def get_token(self) -> str:
         """Gets an access token from Azure AD B2C
         using the token_request_data provided in the constructor."""
         return requests.post(
-            f"https://login.microsoftonline.com/" \
+            f"https://login.microsoftonline.com/"
             f"{self.token_request_data['b2c_tenant_id']}/oauth2/v2.0/token",
             data=self.token_request_data,
             timeout=30).json()['access_token']
 
-
     def get_auth_header(self) -> dict:
         """Returns a dictionary containing the authorization header"""
         return {
@@ -68,7 +93,6 @@ class B2CHelper:
             'Content-type': 'application/json',
         }
 
-
     @staticmethod
     def create_item(auth_header: dict, item: dict) -> requests.Response:
         """Creates a user in Azure AD B2C using the provided item and authorization header."""
@@ -79,7 +103,6 @@ class B2CHelper:
             timeout=30
         )
 
-
     @staticmethod
     def update_item(auth_header: dict, item: dict, item_id: str) -> requests.Response:
         """Updates a user in Azure AD B2C using the provided item and authorization header."""
@@ -91,7 +114,6 @@ class B2CHelper:
             timeout=30
         )
 
-
     @staticmethod
     def delete_item(auth_header: dict, user_id: dict) -> requests.Response:
         """Deletes a user in Azure AD B2C using the provided user_id and authorization header."""
@@ -101,16 +123,24 @@ class B2CHelper:
             timeout=30
         )
 
-
     def get_user(self, user_id: str) -> dict:
         """Gets a user in Azure AD B2C using the provided user_id."""
         user = requests.get(
             f"https://graph.microsoft.com/beta/users/{user_id}", headers=self.get_auth_header(),
             timeout=30
         ).json()
-        del user["@odata.context"]
+
+        if 'error' not in user:
+            del user["@odata.context"]
         return user
 
+    def get_user_by_email(self, email: str) -> dict:
+        """Gets a user in Azure AD B2C using the provided email."""
+
+        return requests.get(
+            f"https://graph.microsoft.com/beta/users?$filter=identities/any(c:c/issuerAssignedId eq '{email}' and c/issuer eq 'eightam72045b2ctenant.onmicrosoft.com')", headers=self.get_auth_header(),
+            timeout=30
+        ).json()
 
     def compile_entire_user_list(self, filter_extension: str) -> list:
         """Compiles the entire list of users in Azure AD B2C using the provided filter_extension."""
@@ -142,36 +172,33 @@ class B2CHelper:
 
         return return_list
 
-
     def get_users(self, company_id: str = None, role_type: str = None) -> list:
         """Gets a list of users in Azure AD B2C using the provided company_id or role_type."""
         if company_id is not None:
             filter_extension = f"?$filter=" \
-            f"extension_{self.token_request_data['extension_id']}_OrganizationID eq '{company_id}'"
+                f"extension_{self.token_request_data['extension_id']}_OrganizationID eq '{company_id}'"
 
             users = self.compile_entire_user_list(filter_extension)
 
         elif role_type is not None:
             filter_extension = f"?$filter=" \
-            f"extension_{self.token_request_data['extension_id']}_UserRoles eq '{role_type}'"
+                f"extension_{self.token_request_data['extension_id']}_UserRoles eq '{role_type}'"
             users = self.compile_entire_user_list(filter_extension)
         else:
             filter_extension = ''
             users = [
-                x for x in self.compile_entire_user_list(filter_extension) \
+                x for x in self.compile_entire_user_list(filter_extension)
                 if x['mailNickname'] != 'Daveg_8amsolutions.com#EXT#'
             ]
 
         return users
 
-
     def create_user(self, user_details: dict, b2c_prefix: str) -> dict:
         """Creates a user in Azure AD B2C using the provided user_details and b2c_prefix."""
         user = build_user_object(
             user_details, self.token_request_data['extension_id'], b2c_prefix)
         return self.create_item(self.get_auth_header(), user)
 
-
     def create_users(self, user_details_list: list, b2c_prefix: str) -> list:
         """Creates a list of users in Azure AD B2C
         using the provided user_details_list and b2c_prefix."""
@@ -183,14 +210,12 @@ class B2CHelper:
             results.append(self.create_item(auth_header, user))
         return results
 
-
     def update_user(self, user_details: dict, b2c_prefix: str) -> dict:
         """Updates a user in Azure AD B2C using the provided user_details and b2c_prefix."""
         user = build_user_object(
             user_details, self.token_request_data['extension_id'], b2c_prefix)
         return self.update_item(self.get_auth_header(), user, user_details['id'])
 
-
     def update_users(self, user_details_list: list, b2c_prefix: str) -> list:
         """Updates a list of users in Azure AD B2C"""
         auth_header = self.get_auth_header()
@@ -202,12 +227,10 @@ class B2CHelper:
                 auth_header, user, user_detail['id']))
         return results
 
-
     def delete_user(self, user_id: str) -> dict:
         """Deletes a user in Azure AD B2C using the provided user_id."""
         return self.delete_item(self.get_auth_header(), user_id)
 
-
     def delete_users(self, user_id_list) -> list:
         """Deletes a list of users in Azure AD B2C using the provided user_id_list."""
         auth_header = self.get_auth_header()
